@@ -26,15 +26,17 @@
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-import contextlib
-import xcffib
-import xcffib.xproto
+from __future__ import annotations
 
-from libqtile.command_object import CommandObject
-from . import hook
-from . import window
-from . import utils
-from .log_utils import logger
+from typing import TYPE_CHECKING
+
+from libqtile import hook, utils
+from libqtile.backend.base import FloatStates
+from libqtile.command.base import CommandObject
+from libqtile.log_utils import logger
+
+if TYPE_CHECKING:
+    from libqtile.command.base import ItemT
 
 
 class _Group(CommandObject):
@@ -45,11 +47,13 @@ class _Group(CommandObject):
 
     A group is identified by its name but displayed in GroupBox widget by its label.
     """
+
     def __init__(self, name, layout=None, label=None):
         self.name = name
         self.label = name if label is None else label
         self.custom_layout = layout  # will be set on _configure
-        self.windows = set()
+        self.windows = []
+        self.tiled_windows = set()
         self.qtile = None
         self.layouts = []
         self.floating_layout = None
@@ -61,12 +65,13 @@ class _Group(CommandObject):
         self.focus_history = []
         self.screen = None
         self.current_layout = None
+        self.last_focused = None
 
     def _configure(self, layouts, floating_layout, qtile):
         self.screen = None
         self.current_layout = 0
         self.focus_history = []
-        self.windows = set()
+        self.windows = []
         self.qtile = qtile
         self.layouts = [i.clone(self) for i in layouts]
         self.floating_layout = floating_layout
@@ -117,23 +122,19 @@ class _Group(CommandObject):
         for index, obj in enumerate(self.layouts):
             if obj.name == layout:
                 self.current_layout = index
-                hook.fire(
-                    "layout_change",
-                    self.layouts[self.current_layout],
-                    self
-                )
+                hook.fire("layout_change", self.layouts[self.current_layout], self)
                 self.layout_all()
                 return
-        raise ValueError("No such layout: %s" % layout)
+        logger.error("No such layout: {}".format(layout))
 
     def use_layout(self, index):
-        assert 0 <= index < len(self.layouts), "layout index out of bounds"
+        assert -len(self.layouts) <= index < len(self.layouts), "layout index out of bounds"
         self.layout.hide()
-        self.current_layout = index
+        self.current_layout = index % len(self.layouts)
         hook.fire("layout_change", self.layouts[self.current_layout], self)
         self.layout_all()
-        screen = self.screen.get_rect()
-        self.layout.show(screen)
+        screen_rect = self.screen.get_rect()
+        self.layout.show(screen_rect)
 
     def use_next_layout(self):
         self.use_layout((self.current_layout + 1) % (len(self.layouts)))
@@ -148,26 +149,25 @@ class _Group(CommandObject):
         to it.
         """
         if self.screen and self.windows:
-            with self.disable_mask(xcffib.xproto.EventMask.EnterWindow):
+            with self.qtile.core.masked():
                 normal = [x for x in self.windows if not x.floating]
-                floating = [
-                    x for x in self.windows
-                    if x.floating and not x.minimized
-                ]
-                screen = self.screen.get_rect()
+                floating = [x for x in self.windows if x.floating and not x.minimized]
+                screen_rect = self.screen.get_rect()
                 if normal:
                     try:
-                        self.layout.layout(normal, screen)
+                        self.layout.layout(normal, screen_rect)
                     except Exception:
-                        logger.exception("Exception in layout %s",
-                                         self.layout.name)
+                        logger.exception("Exception in layout %s", self.layout.name)
                 if floating:
-                    self.floating_layout.layout(floating, screen)
-                if self.current_window and \
-                        self.screen == self.qtile.current_screen:
+                    self.floating_layout.layout(floating, screen_rect)
+                if self.current_window and self.screen == self.qtile.current_screen:
                     self.current_window.focus(warp)
+                else:
+                    # Screen has lost focus so we reset record of focused window so
+                    # focus will warp when screen is focused again
+                    self.last_focused = None
 
-    def _set_screen(self, screen):
+    def set_screen(self, screen, warp=True):
         """Set this group's screen to screen"""
         if screen == self.screen:
             return
@@ -175,30 +175,20 @@ class _Group(CommandObject):
         if self.screen:
             # move all floating guys offset to new screen
             self.floating_layout.to_screen(self, self.screen)
-            self.layout_all(warp=self.qtile.config.cursor_warp)
-            rect = self.screen.get_rect()
-            self.floating_layout.show(rect)
-            self.layout.show(rect)
+            self.layout_all(warp=warp and self.qtile.config.cursor_warp)
+            screen_rect = self.screen.get_rect()
+            self.floating_layout.show(screen_rect)
+            self.layout.show(screen_rect)
         else:
             self.hide()
 
     def hide(self):
         self.screen = None
-        with self.disable_mask(xcffib.xproto.EventMask.EnterWindow |
-                               xcffib.xproto.EventMask.FocusChange |
-                               xcffib.xproto.EventMask.LeaveWindow):
+        with self.qtile.core.masked():
             for i in self.windows:
                 i.hide()
             self.layout.hide()
 
-    @contextlib.contextmanager
-    def disable_mask(self, mask):
-        for i in self.windows:
-            i._disable_mask(mask)
-        yield
-        for i in self.windows:
-            i._reset_mask()
-
     def focus(self, win, warp=True, force=False):
         """Focus the given window
 
@@ -223,7 +213,13 @@ class _Group(CommandObject):
         if win:
             if win not in self.windows:
                 return
+
+            # ignore focus events if window is the current window
+            if win is self.last_focused:
+                warp = False
+
             self.current_window = win
+            self.last_focused = self.current_window
             if win.floating:
                 for layout in self.layouts:
                     layout.blur()
@@ -240,32 +236,28 @@ class _Group(CommandObject):
             name=self.name,
             label=self.label,
             focus=self.current_window.name if self.current_window else None,
+            tiled_windows={i.name for i in self.tiled_windows},
             windows=[i.name for i in self.windows],
             focus_history=[i.name for i in self.focus_history],
             layout=self.layout.name,
-            layouts=[l.name for l in self.layouts],
+            layouts=[i.name for i in self.layouts],
             floating_info=self.floating_layout.info(),
-            screen=self.screen.index if self.screen else None
+            screen=self.screen.index if self.screen else None,
         )
 
     def add(self, win, focus=True, force=False):
-        hook.fire("group_window_add")
-        self.windows.add(win)
+        hook.fire("group_window_add", self, win)
+        if win not in self.windows:
+            self.windows.append(win)
         win.group = self
-        try:
-            if 'fullscreen' in win.window.get_net_wm_state() and \
-                    self.qtile.config.auto_fullscreen:
-                win._float_state = window.FULLSCREEN
-            elif self.floating_layout.match(win):
-                # !!! tell it to float, can't set floating
-                # because it's too early
-                # so just set the flag underneath
-                win._float_state = window.FLOATING
-        except (xcffib.xproto.WindowError, xcffib.xproto.AccessError):
-            pass  # doesn't matter
-        if win.floating:
+        if self.qtile.config.auto_fullscreen and win.wants_to_fullscreen:
+            win._float_state = FloatStates.FULLSCREEN
+        elif self.floating_layout.match(win) and not win.fullscreen:
+            win._float_state = FloatStates.FLOATING
+        if win.floating and not win.fullscreen:
             self.floating_layout.add(win)
-        else:
+        if not win.floating or win.fullscreen:
+            self.tiled_windows.add(win)
             for i in self.layouts:
                 i.add(win)
         if focus:
@@ -279,21 +271,29 @@ class _Group(CommandObject):
         if win.floating:
             nextfocus = self.floating_layout.remove(win)
 
-            nextfocus = nextfocus or \
-                self.current_window or \
-                self.layout.focus_first() or \
-                self.floating_layout.focus_first(group=self)
-        else:
+            nextfocus = (
+                nextfocus
+                or self.current_window
+                or self.layout.focus_first()
+                or self.floating_layout.focus_first(group=self)
+            )
+        # Remove from the tiled layouts if it was not floating or fullscreen
+        if not win.floating or win.fullscreen:
             for i in self.layouts:
                 if i is self.layout:
                     nextfocus = i.remove(win)
                 else:
                     i.remove(win)
 
-            nextfocus = nextfocus or \
-                self.floating_layout.focus_first(group=self) or \
-                self.current_window or \
-                self.layout.focus_first()
+            nextfocus = (
+                nextfocus
+                or self.floating_layout.focus_first(group=self)
+                or self.current_window
+                or self.layout.focus_first()
+            )
+
+            if win in self.tiled_windows:
+                self.tiled_windows.remove(win)
 
         # a notification may not have focus
         if hadfocus:
@@ -310,30 +310,38 @@ class _Group(CommandObject):
                 # already floating
                 pass
             else:
-                for i in self.layouts:
-                    i.remove(win)
+                # Remove from the tiled windows list if the window is not fullscreen
+                if not win.fullscreen:
+                    self.tiled_windows.remove(win)
+                    # Remove the window from the layout if it is not fullscreen
+                    for i in self.layouts:
+                        i.remove(win)
+                        if win is self.current_window:
+                            i.blur()
+                    self.floating_layout.add(win)
                     if win is self.current_window:
-                        i.blur()
-                self.floating_layout.add(win)
-                if win is self.current_window:
-                    self.floating_layout.focus(win)
+                        self.floating_layout.focus(win)
         else:
             self.floating_layout.remove(win)
             self.floating_layout.blur()
-            for i in self.layouts:
-                i.add(win)
-                if win is self.current_window:
+            # A window that was fullscreen should only be added if it was not a tiled window
+            if win not in self.tiled_windows:
+                for i in self.layouts:
+                    i.add(win)
+                self.tiled_windows.add(win)
+            if win is self.current_window:
+                for i in self.layouts:
                     i.focus(win)
         self.layout_all()
 
-    def _items(self, name):
+    def _items(self, name) -> ItemT:
         if name == "layout":
-            return (True, list(range(len(self.layouts))))
-        if name == "screen":
-            return (True, None)
+            return True, list(range(len(self.layouts)))
+        if name == "screen" and self.screen is not None:
+            return True, []
         if name == "window":
-            return (True, [i.window.wid for i in self.windows])
-        raise RuntimeError("Invalid item: {}".format(name))
+            return self.current_window is not None, [i.wid for i in self.windows]
+        return None
 
     def _select(self, name, sel):
         if name == "layout":
@@ -346,7 +354,7 @@ class _Group(CommandObject):
             if sel is None:
                 return self.current_window
             for i in self.windows:
-                if i.window.wid == sel:
+                if i.wid == sel:
                     return i
         raise RuntimeError("Invalid selection: {}".format(name))
 
@@ -357,7 +365,7 @@ class _Group(CommandObject):
         """Returns a dictionary of info for this group"""
         return self.info()
 
-    def cmd_toscreen(self, screen=None, toggle=True):
+    def cmd_toscreen(self, screen=None, toggle=False):
         """Pull a group to a specified screen.
 
         Parameters
@@ -365,7 +373,8 @@ class _Group(CommandObject):
         screen :
             Screen offset. If not specified, we assume the current screen.
         toggle :
-            If this group is already on the screen, then toggle group.
+            If this group is already on the screen, then the group is toggled
+            with last used
 
         Examples
         ========
@@ -400,7 +409,7 @@ class _Group(CommandObject):
         """
 
         def match(group):
-            from . import scratchpad
+            from libqtile import scratchpad
 
             if group is self:
                 return True
@@ -439,13 +448,17 @@ class _Group(CommandObject):
         if not self.windows:
             return
         if self.current_window.floating:
-            nxt = self.floating_layout.focus_next(self.current_window) or \
-                self.layout.focus_first() or \
-                self.floating_layout.focus_first(group=self)
+            nxt = (
+                self.floating_layout.focus_next(self.current_window)
+                or self.layout.focus_first()
+                or self.floating_layout.focus_first(group=self)
+            )
         else:
-            nxt = self.layout.focus_next(self.current_window) or \
-                self.floating_layout.focus_first(group=self) or \
-                self.layout.focus_first()
+            nxt = (
+                self.layout.focus_next(self.current_window)
+                or self.floating_layout.focus_first(group=self)
+                or self.layout.focus_first()
+            )
         self.focus(nxt, True)
 
     def cmd_prev_window(self):
@@ -459,13 +472,17 @@ class _Group(CommandObject):
         if not self.windows:
             return
         if self.current_window.floating:
-            nxt = self.floating_layout.focus_previous(self.current_window) or \
-                self.layout.focus_last() or \
-                self.floating_layout.focus_last(group=self)
+            nxt = (
+                self.floating_layout.focus_previous(self.current_window)
+                or self.layout.focus_last()
+                or self.floating_layout.focus_last(group=self)
+            )
         else:
-            nxt = self.layout.focus_previous(self.current_window) or \
-                self.floating_layout.focus_last(group=self) or \
-                self.layout.focus_last()
+            nxt = (
+                self.layout.focus_previous(self.current_window)
+                or self.floating_layout.focus_last(group=self)
+                or self.layout.focus_last()
+            )
         self.focus(nxt, True)
 
     def cmd_focus_back(self):
