@@ -25,14 +25,11 @@
 # LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
-import inspect
 import os
 
 import cairocffi
 
 from libqtile import bar, hook
-from libqtile import layout as layout_module
-from libqtile.layout.base import Layout
 from libqtile.log_utils import logger
 from libqtile.widget import base
 
@@ -42,28 +39,30 @@ class CurrentLayout(base._TextBox):
     Display the name of the current layout of the current group of the screen,
     the bar containing the widget, is on.
     """
-    orientations = base.ORIENTATION_HORIZONTAL
 
     def __init__(self, width=bar.CALCULATED, **config):
         base._TextBox.__init__(self, "", width, **config)
 
     def _configure(self, qtile, bar):
         base._TextBox._configure(self, qtile, bar)
-        self.text = self.bar.screen.group.layouts[0].name
+        layout_id = self.bar.screen.group.current_layout
+        self.text = self.bar.screen.group.layouts[layout_id].name
         self.setup_hooks()
 
+        self.add_callbacks(
+            {
+                "Button1": qtile.cmd_next_layout,
+                "Button2": qtile.cmd_prev_layout,
+            }
+        )
+
     def setup_hooks(self):
         def hook_response(layout, group):
             if group.screen is not None and group.screen == self.bar.screen:
                 self.text = layout.name
                 self.bar.draw()
-        hook.subscribe.layout_change(hook_response)
 
-    def button_press(self, x, y, button):
-        if button == 1:
-            self.qtile.cmd_next_layout()
-        elif button == 2:
-            self.qtile.cmd_prev_layout()
+        hook.subscribe.layout_change(hook_response)
 
 
 class CurrentLayoutIcon(base._TextBox):
@@ -84,24 +83,20 @@ class CurrentLayoutIcon(base._TextBox):
     - `~/.icons`
     - built-in qtile icons
     """
+
     orientations = base.ORIENTATION_HORIZONTAL
 
     defaults = [
+        ("scale", 1, "Scale factor relative to the bar height.  " "Defaults to 1"),
         (
-            'scale',
-            1,
-            'Scale factor relative to the bar height.  '
-            'Defaults to 1'
-        ),
-        (
-            'custom_icon_paths',
+            "custom_icon_paths",
             [],
-            'List of folders where to search icons before'
-            'using built-in icons or icons in ~/.icons dir.  '
-            'This can also be used to provide'
-            'missing icons for custom layouts.  '
-            'Defaults to empty list.'
-        )
+            "List of folders where to search icons before"
+            "using built-in icons or icons in ~/.icons dir.  "
+            "This can also be used to provide"
+            "missing icons for custom layouts.  "
+            "Defaults to empty list.",
+        ),
     ]
 
     def __init__(self, **config):
@@ -114,7 +109,8 @@ class CurrentLayoutIcon(base._TextBox):
 
     def _configure(self, qtile, bar):
         base._TextBox._configure(self, qtile, bar)
-        self.text = self.bar.screen.group.layouts[0].name
+        layout_id = self.bar.screen.group.current_layout
+        self.text = self.bar.screen.group.layouts[layout_id].name
         self.current_layout = self.text
         self.icons_loaded = False
         self.icon_paths = []
@@ -123,35 +119,36 @@ class CurrentLayoutIcon(base._TextBox):
         self._setup_images()
         self._setup_hooks()
 
+        self.add_callbacks(
+            {
+                "Button1": qtile.cmd_next_layout,
+                "Button2": qtile.cmd_prev_layout,
+            }
+        )
+
     def _setup_hooks(self):
         """
         Listens for layout change and performs a redraw when it occurs.
         """
+
         def hook_response(layout, group):
             if group.screen is not None and group.screen == self.bar.screen:
                 self.current_layout = layout.name
                 self.bar.draw()
-        hook.subscribe.layout_change(hook_response)
 
-    def button_press(self, x, y, button):
-        if button == 1:
-            self.qtile.cmd_next_layout()
-        elif button == 2:
-            self.qtile.cmd_prev_layout()
+        hook.subscribe.layout_change(hook_response)
 
     def draw(self):
         if self.icons_loaded:
             try:
                 surface = self.surfaces[self.current_layout]
             except KeyError:
-                logger.error('No icon for layout {}'.format(
-                    self.current_layout
-                ))
+                logger.error("No icon for layout {}".format(self.current_layout))
             else:
                 self.drawer.clear(self.background or self.bar.background)
                 self.drawer.ctx.set_source(surface)
                 self.drawer.ctx.paint()
-                self.drawer.draw(offsetx=self.offset, width=self.length)
+                self.drawer.draw(offsetx=self.offset, offsety=self.offsety, width=self.length)
         else:
             # Fallback to text
             self.text = self.current_layout[0].upper()
@@ -161,12 +158,7 @@ class CurrentLayoutIcon(base._TextBox):
         """
         Returns the list of lowercased strings for each available layout name.
         """
-        return [
-            layout_class_name.lower()
-            for layout_class, layout_class_name
-            in map(lambda x: (getattr(layout_module, x), x), dir(layout_module))
-            if inspect.isclass(layout_class) and issubclass(layout_class, Layout)
-        ]
+        return [layout.__class__.__name__.lower() for layout in self.qtile.config.layouts]
 
     def _update_icon_paths(self):
         self.icon_paths = []
@@ -174,17 +166,18 @@ class CurrentLayoutIcon(base._TextBox):
         # We allow user to override icon search path
         self.icon_paths.extend(self.custom_icon_paths)
 
-        # We also look in ~/.icons/
-        self.icon_paths.append(os.path.expanduser('~/.icons'))
+        # We also look in ~/.icons/ and ~/.local/share/icons
+        self.icon_paths.append(os.path.expanduser("~/.icons"))
+        self.icon_paths.append(os.path.expanduser("~/.local/share/icons"))
 
         # Default icons are in libqtile/resources/layout-icons.
         # If using default config without any custom icons,
         # this path will be used.
         root = os.sep.join(os.path.abspath(__file__).split(os.sep)[:-2])
-        self.icon_paths.append(os.path.join(root, 'resources', 'layout-icons'))
+        self.icon_paths.append(os.path.join(root, "resources", "layout-icons"))
 
     def find_icon_file_path(self, layout_name):
-        icon_filename = 'layout-{}.png'.format(layout_name)
+        icon_filename = "layout-{}.png".format(layout_name)
         for icon_path in self.icon_paths:
             icon_file_path = os.path.join(icon_path, icon_filename)
             if os.path.isfile(icon_file_path):
@@ -198,7 +191,7 @@ class CurrentLayoutIcon(base._TextBox):
             icon_file_path = self.find_icon_file_path(layout_name)
             if icon_file_path is None:
                 logger.warning('No icon found for layout "{}"'.format(layout_name))
-                icon_file_path = self.find_icon_file_path('unknown')
+                icon_file_path = self.find_icon_file_path("unknown")
 
             try:
                 img = cairocffi.ImageSurface.create_from_png(icon_file_path)
@@ -209,7 +202,7 @@ class CurrentLayoutIcon(base._TextBox):
                 self.icons_loaded = False
                 logger.exception(
                     'Failed to load icon from file "{}", '
-                    'error was: {}'.format(icon_file_path, e.message)
+                    "error was: {}".format(icon_file_path, e.message)
                 )
                 return
 
