@@ -40,35 +40,36 @@ class Mpris2(base._TextBox):
 
     .. _dbus-next: https://pypi.org/project/dbus-next/
     """
-    orientations = base.ORIENTATION_HORIZONTAL
+
     defaults = [
-        ('name', 'audacious', 'Name of the MPRIS widget.'),
-
-        ('objname', 'org.mpris.MediaPlayer2.audacious',
-            'DBUS MPRIS 2 compatible player identifier'
-            '- Find it out with dbus-monitor - '
-            'Also see: http://specifications.freedesktop.org/'
-            'mpris-spec/latest/#Bus-Name-Policy'),
-
-        ('display_metadata', ['xesam:title', 'xesam:album', 'xesam:artist'],
-            'Which metadata identifiers to display. '
-            'See http://www.freedesktop.org/wiki/Specifications/mpris-spec/metadata/#index5h3 '
-            'for available values'),
-
-        ('scroll_chars', 30, 'How many chars at once to display.'),
-        ('scroll_interval', 0.5, 'Scroll delay interval.'),
-        ('scroll_wait_intervals', 8, 'Wait x scroll_interval before'
-            'scrolling/removing text'),
-
-        ('stop_pause_text', None, "Optional text to display when in the stopped/paused state"),
+        ("name", "audacious", "Name of the MPRIS widget."),
+        (
+            "objname",
+            "org.mpris.MediaPlayer2.audacious",
+            "DBUS MPRIS 2 compatible player identifier"
+            "- Find it out with dbus-monitor - "
+            "Also see: http://specifications.freedesktop.org/"
+            "mpris-spec/latest/#Bus-Name-Policy",
+        ),
+        (
+            "display_metadata",
+            ["xesam:title", "xesam:album", "xesam:artist"],
+            "Which metadata identifiers to display. "
+            "See http://www.freedesktop.org/wiki/Specifications/mpris-spec/metadata/#index5h3 "
+            "for available values",
+        ),
+        ("scroll_chars", 30, "How many chars at once to display."),
+        ("scroll_interval", 0.5, "Scroll delay interval."),
+        ("scroll_wait_intervals", 8, "Wait x scroll_interval before" "scrolling/removing text"),
+        ("stop_pause_text", None, "Optional text to display when in the stopped/paused state"),
     ]
 
     def __init__(self, **config):
-        base._TextBox.__init__(self, '', **config)
+        base._TextBox.__init__(self, "", **config)
         self.add_defaults(Mpris2.defaults)
 
         self.scrolltext = None
-        self.displaytext = ''
+        self.displaytext = ""
         self.is_playing = False
         self.scroll_timer = None
         self.scroll_counter = None
@@ -76,12 +77,13 @@ class Mpris2(base._TextBox):
 
     async def _config_async(self):
         subscribe = await add_signal_receiver(
-                        self.message,
-                        session_bus=True,
-                        signal_name="PropertiesChanged",
-                        bus_name=self.objname,
-                        path="/org/mpris/MediaPlayer2",
-                        dbus_interface="org.freedesktop.DBus.Properties")
+            self.message,
+            session_bus=True,
+            signal_name="PropertiesChanged",
+            bus_name=self.objname,
+            path="/org/mpris/MediaPlayer2",
+            dbus_interface="org.freedesktop.DBus.Properties",
+        )
 
         if not subscribe:
             msg = "Unable to add signal receiver for {}.".format(self.objname)
@@ -94,58 +96,58 @@ class Mpris2(base._TextBox):
         self.update(*message.body)
 
     def update(self, interface_name, changed_properties, _invalidated_properties):
-        """http://specifications.freedesktop.org/mpris-spec/latest/Track_List_Interface.html#Mapping:Metadata_Map"""
+        """
+        http://specifications.freedesktop.org/mpris-spec/latest/Track_List_Interface.html#Mapping:Metadata_Map
+        """
         if not self.configured:
             return True
         olddisplaytext = self.displaytext
-        self.displaytext = ''
+        self.displaytext = ""
 
-        metadata = changed_properties.get('Metadata')
+        metadata = changed_properties.get("Metadata")
         if metadata:
             metadata = metadata.value
             self.is_playing = True
 
             meta_list = []
             for key in self.display_metadata:
-                val = getattr(metadata.get(key), 'value', None)
+                val = getattr(metadata.get(key), "value", None)
                 if isinstance(val, str):
                     meta_list.append(val)
                 elif isinstance(val, list):
                     val = " - ".join((y for y in val if isinstance(y, str)))
                     meta_list.append(val)
 
-            self.displaytext = ' - '.join(meta_list)
-            self.displaytext.replace('\n', '')
+            self.displaytext = " - ".join(meta_list)
+            self.displaytext.replace("\n", "")
 
-        playbackstatus = getattr(changed_properties.get('PlaybackStatus'),
-                                 'value',
-                                 None)
-        if playbackstatus == 'Paused':
+        playbackstatus = getattr(changed_properties.get("PlaybackStatus"), "value", None)
+        if playbackstatus == "Paused":
             if self.stop_pause_text is not None:
                 self.is_playing = False
                 self.displaytext = self.stop_pause_text
             elif self.displaytext:
                 self.is_playing = False
-                self.displaytext = 'Paused: {}'.format(self.displaytext)
+                self.displaytext = "Paused: {}".format(self.displaytext)
             else:
                 self.is_playing = False
-                self.displaytext = 'Paused'
-        elif playbackstatus == 'Playing':
+                self.displaytext = "Paused"
+        elif playbackstatus == "Playing":
             if not self.displaytext and olddisplaytext:
                 self.is_playing = True
-                self.displaytext = olddisplaytext.replace('Paused: ', '')
+                self.displaytext = olddisplaytext.replace("Paused: ", "")
             elif not self.displaytext and not olddisplaytext:
                 self.is_playing = True
-                self.displaytext = 'No metadata for current track'
+                self.displaytext = "No metadata for current track"
             elif self.displaytext:
                 # Players might send more than one "Playing" message.
                 pass
         elif playbackstatus:
             self.is_playing = False
-            self.displaytext = ''
+            self.displaytext = ""
 
         if self.scroll_chars and self.scroll_interval:
-            if(self.scroll_timer):
+            if self.scroll_timer:
                 self.scroll_timer.cancel()
             self.scrolltext = self.displaytext
             self.scroll_counter = self.scroll_wait_intervals
@@ -156,8 +158,8 @@ class Mpris2(base._TextBox):
             self.bar.draw()
 
     def scroll_text(self):
-        if self.text != self.scrolltext[:self.scroll_chars]:
-            self.text = self.scrolltext[:self.scroll_chars]
+        if self.text != self.scrolltext[: self.scroll_chars]:
+            self.text = self.scrolltext[: self.scroll_chars]
             self.bar.draw()
         if self.scroll_counter:
             self.scroll_counter -= 1
@@ -170,7 +172,7 @@ class Mpris2(base._TextBox):
                 self.scroll_counter += self.scroll_wait_intervals
             self.timeout_add(self.scroll_interval, self.scroll_text)
             return
-        self.text = ''
+        self.text = ""
         self.bar.draw()
 
     def cmd_info(self):
